//===============================================================
// vCanvas - a scrolling window canvas for drawing
//
// Copyright (C) 1995,1996  Bruce E. Wampler
//
// This file is part of the V C++ GUI Framework, and is covered
// under the terms of the GNU Library General Public License,
// Version 2. This library has NO WARRANTY. See the source file
// vapp.cxx for more complete information about license terms.
//===============================================================

#include <v/vcanvas.h>		// our header
/*
#include <v/vctlclrs.h>
*/

#ifndef GTK_HAVE_FEATURES_1_1_6
#define GTK_POLICY_NEVER GTK_POLICY_AUTOMATIC
#endif

/*
extern "C"
{
#ifdef Motif
#include <Xm/ScrollBar.h>
#include <Xm/DrawingA.h>
#define sbWidgetClass xmScrollBarWidgetClass
#define sbThickness 15
#else
#include <X11/Xaw/Paned.h>

#ifdef USE3D
#include <v/awscl3d.h>
#define sbWidgetClass scrollbar3dWidgetClass
#define sbThickness 15
#else
#include <X11/Xaw/Scrollbar.h>
#define sbWidgetClass scrollbarWidgetClass
#define sbThickness 10
#endif
#endif

#include <v/canvas.h>		// our own canvas widget

}
*/
#include <v/vapp.h>		// need access to the app
#include <v/vwindow.h>		// we are part of a vWindow
#include <v/vcpdc.h>		// for vCanvasPaneDC

    VCursor vCanvasPane::_currentCursor = -1;	// for cursor

// Put the X dependent Cursor tables here...

/*
   static unsigned int VtoXmap[] =
      {
	XC_X_cursor,  XC_top_left_arrow, XC_center_ptr, XC_crosshair,
	XC_sb_h_double_arrow, XC_hand2, XC_xterm, XC_icon,
	XC_sb_v_double_arrow, XC_pencil, XC_question_arrow,
	XC_sizing, XC_watch, XC_X_cursor
      };
*/
   static enum GdkCursorType VtoGdkmap[] =
      {
	GDK_X_CURSOR, GDK_TOP_LEFT_ARROW, GDK_CENTER_PTR, GDK_CROSSHAIR,
	GDK_SB_H_DOUBLE_ARROW, GDK_HAND2, GDK_XTERM, GDK_ICON,
	GDK_SB_V_DOUBLE_ARROW, GDK_PENCIL, GDK_QUESTION_ARROW,
	GDK_SIZING, GDK_WATCH, GDK_X_CURSOR
      };

/*
    static Cursor XCursors[maxCursors+1];
*/
    static GdkCursor *cursors[maxCursors+1];

extern "C" 
{
    static void CFocusIn(GtkWidget *widget, GdkEventFocus *event);
    static void CFocusOut(GtkWidget *widget, GdkEventFocus *event);
}
//================>>> vCanvasPane::vCanvasPane <<<========================
  vCanvasPane::vCanvasPane(PaneType pt) : vPane(pt)
  {
    SysDebug(Constructor,"vCanvasPane::vCanvasPane() constructor\n")

    _cpDC = 0;			// no pane DC
    _compressRedraw = 1;	// compress by default

    if (_currentCursor == -1)		// Do we need to create cursors?
      {
	_currentCursor = VC_None;
	for (int i = 0 ; i <= maxCursors ; ++i)  // Create all cursors
	  {
	    cursors[i] = gdk_cursor_new(VtoGdkmap[i]);
/*
	    XCursors[i] = XCreateFontCursor(theApp->display(),VtoXmap[i]);
*/
	    // Get the fg and bg to make cursor match
	  }
      }

  }

//================>>> vCanvasPane::~vCanvasPane <<<========================
  vCanvasPane::~vCanvasPane()
  {
    SysDebug(Destructor,"vCanvasPane::~vCanvasPane() destructor\n")
    if (_cpDC != 0)
	delete _cpDC;	// free the DC
  }

//==================>>> vCanvasPane::initialize <<<==========================
  void vCanvasPane::initialize(vWindow* pWindow, Widget pWidget, 
	Widget drawCanvas, Widget drawArea)
  {
/*
    XtCallbackRec HscrollProcList[] =
      {
	{ CHScrollProcCB, this },
	{ (XtCallbackProc) 0, (XtPointer) 0}
      };
    XtCallbackRec VscrollProcList[] =
      {
	{ CVScrollProcCB, this },
	{ (XtCallbackProc) 0, (XtPointer) 0}
      };

#ifndef Motif		// ATHENA
    XtCallbackRec HjumpProcList[] =
      {
	{ CHJumpProcCB, this },
	{ (XtCallbackProc) 0, (XtPointer) 0}
      };

    XtCallbackRec VjumpProcList[] =
      {
	{ CVJumpProcCB, this },
	{ (XtCallbackProc) 0, (XtPointer) 0}
      };
#endif
*/

    SysDebug(Constructor,"vCanvasPane::initialize()\n")
    _ResizeDone = 0;
    _HScrlShown = 100;	//@@@@
    _HScrlTop =  0;
    _VScrlShown = 100;
    _VScrlTop =  0;

     // now, build the menu items in the widget provided

    vPane::initialize(pWindow,pWidget);	// initialize base class

    // canvasWidgetClass is V's own Widget that manages a drawing
    // area and scroll bars.

    if (drawCanvas != 0)	// canvas provided by subclass
      {
	_drawCanvas = drawCanvas;
      }
    else
      {
	_drawCanvas = gtk_table_new(2, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(_baseW), _drawCanvas, TRUE, TRUE, 0);
	gtk_widget_show(_drawCanvas);
      }


    pWindow->setworkArea(_drawCanvas);
     
    // --------------------------------------------------------------
    _hasFocus = 0;	// we don't start out with focus
    _mouseDown = 0;	// mouse is not down

/*
    Dimension curW, curH;
*/
    gint curW, curH;

    if (drawArea != 0)
      {
	_drawWindow = drawArea;
      }
    else
      {
	_drawWindow = gtk_drawing_area_new();

	/* XXX what if drawArea != 0 */
	/* needs to be called before realization */
	gtk_widget_set_events(_drawWindow, GDK_EXPOSURE_MASK
		| GDK_BUTTON_PRESS_MASK
		| GDK_BUTTON_RELEASE_MASK
		| GDK_KEY_PRESS
		| GDK_POINTER_MOTION_MASK
		| GDK_POINTER_MOTION_HINT_MASK
		| GDK_ENTER_NOTIFY_MASK
		| GDK_LEAVE_NOTIFY_MASK);

	gtk_drawing_area_size(GTK_DRAWING_AREA(_drawWindow), 
	    _parentWin->_WinWidth, _parentWin->_WinHeight);
	gtk_table_attach_defaults(GTK_TABLE(_drawCanvas), _drawWindow,
	    0, 1, 0, 1);
	gtk_widget_show(_drawWindow);
      }

    GtkStyle *style = gtk_style_new();
    style->bg[GTK_STATE_NORMAL] = *vStdColors[vC_White].color();
    gtk_widget_set_style(_drawWindow, style);

    // --------------------------------------------------------------

    // After the drawing widget has been created, we can create
    // the DC for it to use.

    CreateDC();

    // now create scroll bars, add to the _drawCanvas

    _vadj = (GtkAdjustment*) gtk_adjustment_new(0, 0, 100, 1, 10, 5);
    gtk_signal_connect(GTK_OBJECT(_vadj), "value_changed",
	GTK_SIGNAL_FUNC(CVJumpProcCB), (gpointer)this);
    _drawVScroll = gtk_vscrollbar_new(_vadj);
    gtk_table_attach(GTK_TABLE(_drawCanvas), _drawVScroll, 1, 2, 0, 1, 
	(GtkAttachOptions)0, GTK_FILL, 0, 0);
    _hadj = (GtkAdjustment*) gtk_adjustment_new(0, 0, 100, 1, 10, 5);
    gtk_signal_connect(GTK_OBJECT(_hadj), "value_changed",
	GTK_SIGNAL_FUNC(CHJumpProcCB), (gpointer)this);
    _drawHScroll = gtk_hscrollbar_new(_hadj);
    gtk_table_attach(GTK_TABLE(_drawCanvas), _drawHScroll, 0, 1, 1, 2, 
	 GTK_FILL,(GtkAttachOptions)0, 0, 0);

    // handle required events - requires redrawing

    gtk_signal_connect(GTK_OBJECT(_drawWindow), "configure_event",
	GTK_SIGNAL_FUNC(CchangeCB), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "expose_event",
	GTK_SIGNAL_FUNC(CExposeEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "button_press_event",
	GTK_SIGNAL_FUNC(CMouseDownEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "motion_notify_event",
	GTK_SIGNAL_FUNC(CMouseMoveEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "button_release_event",
	GTK_SIGNAL_FUNC(CMouseUpEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "enter_notify_event",
	GTK_SIGNAL_FUNC(CEnterEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "leave_notify_event",
	GTK_SIGNAL_FUNC(CLeaveEV), (gpointer)this);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "focus_in_event",
	GTK_SIGNAL_FUNC(CFocusIn), NULL);
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "focus_out_event",
	GTK_SIGNAL_FUNC(CFocusOut), NULL);


    gtk_signal_connect(GTK_OBJECT(_parentWin->_topLevel), "key_press_event",
	GTK_SIGNAL_FUNC(CKeyInEV), (gpointer)_parentWin);
    /*
    gtk_signal_connect(GTK_OBJECT(_drawWindow), "key_press_event",
	GTK_SIGNAL_FUNC(CKeyInEV), (gpointer)_parentWin);
    GTK_WIDGET_SET_FLAGS(_drawWindow, GTK_CAN_FOCUS);
    gtk_widget_grab_focus(_drawWindow);
    */

    /*
    gtk_signal_connect(GTK_OBJECT(_parentWin->_vHandle), "set-focus-child",
	GTK_SIGNAL_FUNC(CSetFocusChild), (gpointer) this);
    */

    _HOn = _VOn = 0;			// both start out off

    SetHScroll(_HScrlShown, 0);	// initial values for scroll bar
    SetVScroll(_VScrlShown, 0);

/*
    XtUnmanageChild(_drawHScroll);	// Unmanage the widget
    XtUnmanageChild(_drawVScroll);	// Unmanage the widget

    XtVaGetValues(_drawCanvas,		// get values for the draw canvas
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _height = curH;	// update the local stuff
    _width = curW;
*/

    _height = _parentWin->_WinHeight;	// update the local stuff
    _width = _parentWin->_WinWidth;

/*
    // The parent will resize based on stuff the child does.

    XtVaGetValues(_baseW,			// parent widget
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _parentWin->_WinWidth = curW;
    _parentWin->_WinHeight = curH;	// update the local stuff
*/

    _compressRedraw = 1;       // compress by default
    _Drawable = 0;
  }

//==================>>> vCanvasPane::CreateDC <<<========================
  void vCanvasPane::CreateDC(void)
  {
    if (!_cpDC)
	_cpDC = new vCanvasPaneDC(this);	// create a CanvasPaneDC
  }

//==================>>> vCanvasPane::ChangeCB <<<========================
  void vCanvasPane::ChangeCB(int width, int height)
  {
    _height = height;
    _width = width;

    // The parent will resize based on stuff the child does.

    int pW, pH;
    gdk_window_get_size(_baseW->window, &pW, &pH);
    _parentWin->_WinWidth = pW;
    _parentWin->_WinHeight = pH;	// update the local stuff

    Resize(width, height);

  }

//====================>>> vCanvasPane::ShowPane <<<======================
  void vCanvasPane::ShowPane(int OnOrOff) VCONST
  {
    if (OnOrOff)
      {
	gtk_widget_show(_drawCanvas);
      }
    else
      {
	gtk_widget_hide(_drawCanvas);
      }
  }

//=====================>>> vCanvasPane::SetWidthHeight <<<============================
  void vCanvasPane::SetWidthHeight(int width, int height)
  {
    int curW = width;
    int curH = height;

    SysDebug(Constructor,"vCanvasPane::setWidthHeight\n")

    gtk_drawing_area_size(GTK_DRAWING_AREA(_drawWindow), curW, curH);
    _height = curH;
    _width = curW;

/*
    Dimension curW = width;
    Dimension curH = height;
    Dimension oldbaseW;
    Dimension oldbaseH;
    Dimension oldcnvW;
    Dimension oldcnvH;

    // find out the size difference between the window and the canvas

    XtVaGetValues(_baseW,			// parent widget
	XtNwidth, &oldbaseW,		// its current h and w
	XtNheight, &oldbaseH,
	NULL);

    XtVaGetValues(_drawCanvas,			// parent widget
	XtNwidth, &oldcnvW,		// its current h and w
	XtNheight, &oldcnvH,
	NULL);

    Dimension DiffW = (oldcnvW <= oldbaseW) ? oldbaseW - oldcnvW : 0;
    Dimension DiffH = (oldcnvH <= oldbaseH) ? oldbaseH - oldcnvH : 0;

    XtVaSetValues(_baseW,		// get values for the draw canvas
	XtNwidth, curW + DiffW,		// its new h and w
	XtNheight, curH + DiffH,	// its new h and w
	NULL);

    // Now set the height of the canvas window. You don't always
    // get what you asked for, however.
    XtVaSetValues(_drawCanvas,	// get values for the draw canvas
	XtNheight, curH,
	XtNwidth, curW,		// its new h and w
	NULL);

    // Get these from X -- the window manager can muck these up.
    XtVaGetValues(_drawCanvas,		// get values for the draw canvas
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _height = curH;
    _width = curW;

    XtVaGetValues(_baseW,			// parent widget
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _parentWin->_WinWidth = curW;
    _parentWin->_WinHeight = curH;
*/

  }

//------------------------- cursor Stuff ---------------------------------
//=====================>>> vCanvasPane::SetCursor <<<============================
  void vCanvasPane::SetCursor(VCursor id)
  {
    // Set to current cursor

    if (id < VC_None || id > VC_LAST)
      {
	SysDebug1(BadVals,"vCanvasPane::SetCursor(id=%d)\n",id)
	return;
      }

    if (id == VC_None)		// special case
	UnSetCursor();

    SysDebug1(WindowEvents,"vCanvasPane::SetCursor(id=%d)\n",id)

    _currentCursor = id;

    gdk_window_set_cursor(_drawWindow->window, cursors[id]);
/*
    XDefineCursor(theApp->display(), XtWindow(_drawWindow), XCursors[id]);
*/
  }

//=====================>>> vCanvasPane::UnSetCursor <<<============================
  void vCanvasPane::UnSetCursor(void)
  {
/*
    XUndefineCursor(theApp->display(), XtWindow(_drawWindow));
*/
    _currentCursor = VC_None;
  }

//------------------------- Scrolling Stuff ---------------------------------
//====================>>> vCanvasPane::HJumpProcCB <<<=========================
  void vCanvasPane::HJumpProcCB(float fpercent)
  {
    // This is called with a fraction from 0 to 1.0 indicating
    // how far the user has moved the scroll bar left or right.
    // I can't get the Athena scroll widget to work how I want, so
    // I'll live with how it seems to want to work: the shown
    // fraction only is "correct" when it will all fit. The top
    // will always be right - but 100 is at the bottom with the
    // shown drawn out of the scroll box. Sigh...

    int shown = _HScrlShown;
    int top = (int)fpercent;

    if (top != _HScrlTop)	// actually changed things
      {
	HPage(shown, top);
      }
  }

//=======================>>> vCanvasPane::HPage <<<============================
  void vCanvasPane::HPage(int Shown, int Top)
  {
    // This is the main way a user app gets scrolling events, and
    // will usually override this guy to handle what is actually
    // drawn in the canvas window.
    // This should then be called to actually change the display.

    SysDebug2(WindowEvents,"vCanvasPane::HPage(Shown: %d, Top: %d)\n",Shown, Top);

    SetHScroll(Shown,Top);
  }

//=======================>>> vCanvasPane::HScroll <<<==========================
  void vCanvasPane::HScroll(int step)
  {
    // This is the way the user app gets step events - either page or
    // single step events. The default doesn't have to do anything.

    SysDebug1(WindowEvents,"vCanvasPane::HScroll(%d)\n",step);

  }

//=====================>>> vCanvasPane::ShowHScroll <<<=========================
  void vCanvasPane::ShowHScroll(int OnOff)
  {
    if (OnOff)
	gtk_widget_show(_drawHScroll);
    else
	gtk_widget_hide(_drawHScroll);

    GtkRequisition r;
    gtk_widget_size_request(GTK_WIDGET(_drawCanvas), &r);
/*
    if (OnOff)			// Turn on
      {
	if (_HOn)		// Already on
	    return;
	_HOn = 1;
	XtManageChild(_drawHScroll);	// Unmanage the widget
      }
    else			// Turn off
      {
	if (!_HOn)		// Already off
	    return;
	_HOn = 0;
	XtUnmanageChild(_drawHScroll);	// Unmanage the widget
      }

    Dimension curW, curH;

    XtVaGetValues(_drawWindow,		// get values for the draw canvas
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _height = curH;	// update the local stuff
    _width = curW;

    Dimension pW, pH;
    XtVaGetValues(_baseW,			// parent widget
	XtNwidth, &pW,		// its new h and w
	XtNheight, &pH,
	NULL);

    _parentWin->_WinWidth = pW;
    _parentWin->_WinHeight = pH;	// update the local stuff

    XClearArea(XtDisplay(_drawWindow), XtWindow(_drawWindow),
	0, 0, _width, _height, 1);
*/

  }

//=====================>>> vCanvasPane::ShowVScroll <<<=========================
  void vCanvasPane::ShowVScroll(int OnOff)
  {
    if (OnOff)
	gtk_widget_show(_drawVScroll);
    else
	gtk_widget_hide(_drawVScroll);

    GtkRequisition r;
    gtk_widget_size_request(GTK_WIDGET(_drawCanvas), &r);
/*
    if (OnOff)			// Turn on
      {
	if (_VOn)		// Already on
	    return;
	_VOn = 1;
	XtManageChild(_drawVScroll);	// Unmanage the widget
      }
    else			// Turn off
      {
	if (!_VOn)		// Already off
	    return;
	_VOn = 0;
	XtUnmanageChild(_drawVScroll);	// Unmanage the widget
      }

    Dimension curW, curH;

    XtVaGetValues(_drawWindow,		// get values for the draw canvas
	XtNwidth, &curW,		// its new h and w
	XtNheight, &curH,
	NULL);

    _height = curH;	// update the local stuff
    _width = curW;

    Dimension pW, pH;
    XtVaGetValues(_baseW,			// parent widget
	XtNwidth, &pW,		// its new h and w
	XtNheight, &pH,
	NULL);

    _parentWin->_WinWidth = pW;
    _parentWin->_WinHeight = pH;	// update the local stuff

    XClearArea(XtDisplay(_drawWindow), XtWindow(_drawWindow),
	0, 0, _width, _height, 1);
*/
  }

//=====================>>> vCanvasPane::SetHScroll <<<=========================
  void vCanvasPane::SetHScroll(int Shown, int Top)
  {
    // Make sure we are setting to valid values, and are changing things!

    if (Shown < 0 || Shown > 100 || Top < 0 || Top > 100)
	return;

    _HScrlShown = Shown;
    if (Shown < 5)		// it must be something that shows
	_HScrlShown = 5;

    if (Shown == 100)		// Map top to space we have available
	_HScrlTop = 0;
    else
	_HScrlTop = Top;

    DrawHScroll(_HScrlShown, _HScrlTop);

  }

//=====================>>> vCanvasPane::DrawHScroll <<<========================
  void vCanvasPane::DrawHScroll(int Shown, int Top)
  {
    _hadj->page_size = Shown;
    _hadj->value = Top;
    gtk_adjustment_changed(_hadj);
    gtk_adjustment_value_changed(_hadj);
/*
    Arg args[3];		// Used to pass float value

#ifdef Motif
    float shown = (float) Shown;
    int iTop;

    if (Top == 0)
	iTop = 0;
    else if (Top == 100)
	iTop = 100 - Shown;
    else
	iTop = (int) ((100. - shown)*((float)Top / 100.));

    XtVaSetValues(_drawHScroll,
	XmNsliderSize,Shown,
	XmNvalue,iTop,
	NULL);
#else
    union
      {
	XtArgVal arg_value;
	float float_value;
      } shown_value, top_value;

    float shown = float(Shown) / 100.;
    float top;

    if (Top == 0)
	top = 0.;
    else
	top = float(Top) / 100.;
//1.20	top = (1. - shown)*(float(Top) / 100.);

    shown_value.float_value = shown;
    top_value.float_value = top;

    if (sizeof (float) > sizeof (XtArgVal))
      {
	XtSetArg(args[0], XtNtopOfThumb, &top_value);
	XtSetArg(args[1], XtNshown, &shown_value);
      }
    else
      {
	XtSetArg(args[0], XtNtopOfThumb, top_value.arg_value);
	XtSetArg(args[1], XtNshown, shown_value.arg_value);
      }

    XtSetValues(_drawHScroll, args, 2);		// change it!
#endif
*/

  }

//====================>>> vCanvasPane::VJumpProcCB <<<=======================
  void vCanvasPane::VJumpProcCB(float fpercent)
  {
    int shown = _VScrlShown;
    int top = (int)fpercent;

    if (top != _VScrlTop)	// actually changed things
      {
	VPage(shown, top);
      }
  }

//========================>>> vCanvasPane::VPage <<<===========================
  void vCanvasPane::VPage(int Shown, int Top)
  {

    SysDebug2(WindowEvents,"vCanvasPane::VPage(Shown: %d, Top: %d)\n",Shown, Top);

    SetVScroll(Shown,Top);

  }

//========================>>> vCanvasPane::VScroll <<<=========================
  void vCanvasPane::VScroll(int step)
  {
    // This is the way the user app gets step events - either page or
    // single step events. The default doesn't have to do anything.

    SysDebug1(WindowEvents,"vCanvasPane::VScroll(%d)\n",step);

  }

//=====================>>> vCanvasPane::GetVScroll <<<=========================
  int vCanvasPane::GetVScroll(int& Shown, int& Top) VCONST
  {
    Shown = _VScrlShown; Top = _VScrlTop;
    return _VOn;
  }

//=====================>>> vCanvasPane::GetHScroll <<<=========================
  int vCanvasPane::GetHScroll(int& Shown, int& Top) VCONST
  {
    Shown = _HScrlShown; Top = _HScrlTop;
    return _HOn;
  }

//=====================>>> vCanvasPane::SetVScroll <<<=========================
  void vCanvasPane::SetVScroll(int Shown, int Top)
  {
    // Make sure we are setting to valid values, and are changing things!

    if (Shown < 0 || Shown > 100 || Top < 0 || Top > 100)
	return;

    _VScrlShown = Shown;
    if (Shown < 5)
	_VScrlShown = 5;

    if (Shown == 100)		// Map top to space we have available
	_VScrlTop = 0;
    else
	_VScrlTop = Top;

    DrawVScroll(_VScrlShown, _VScrlTop);
  }

//======================>>> vCanvasPane::DrawVScroll <<<=======================
  void vCanvasPane::DrawVScroll(int Shown, int Top)
  {
    _vadj->page_size = Shown;
    _vadj->value = Top;
    gtk_adjustment_changed(_vadj);
    gtk_adjustment_value_changed(_vadj);

/*
#ifdef Motif
    float shown = (float) Shown;
    int iTop;

    if (Top == 0)
	iTop = 0;
    else if (Top == 100)
	iTop = 100 - Shown;
    else
	iTop = (int) ((100. - shown)*((float)Top / 100.));
	
    XtVaSetValues(_drawVScroll,
	XmNsliderSize,Shown,
	XmNvalue,iTop,
	NULL);
#else
    Arg args[3];		// Used to pass float value

    union
      {
	XtArgVal arg_value;
	float float_value;
      } shown_value, top_value;

    float shown = float(Shown) / 100.;
    float top;


    if (Top == 0)
	top = 0.;
    else
	top = float(Top) / 100.;
//1.20	 top = (1. - shown)*(float(Top) / 100.);

    shown_value.float_value = shown;
    top_value.float_value = top;

    if (sizeof (float) > sizeof (XtArgVal))
      {
	XtSetArg(args[0], XtNtopOfThumb, &top_value);
	XtSetArg(args[1], XtNshown, &shown_value);
      }
    else
      {
	XtSetArg(args[0], XtNtopOfThumb, top_value.arg_value);
	XtSetArg(args[1], XtNshown, shown_value.arg_value);
      }

    XtSetValues(_drawVScroll, args, 2);		// change it!
#endif
*/

  }

//====================>>> vCanvasPane::HScrollProcCB <<<=======================
  void vCanvasPane::HScrollProcCB(int position)
  {
    if (position  > 0)		// Scroll right a page
	HScroll(1);
    else
	HScroll(-1);		// Scroll left a page
  }

//====================>>> vCanvasPane::VScrollProcCB <<<=======================
  void vCanvasPane::VScrollProcCB(int position)
  {
    if (position  > 0)		// scroll list down one page
	VScroll(1);
    else
	VScroll(-1);		// scroll list up one page
  }

//=====================>>> vCanvasPane::ExposeEV <<<==========================
  void vCanvasPane::ExposeEV(int x, int y, int width, int height, int count)
  {
    // We need to be a little intelligent about handling expose
    // events. Most users don't care about getting all the little
    // rectangles that are generated by dragging a window. Most users
    // just redraw everything on each Redraw, which can be very ugly
    // if you get a bunch of little rectangles in a row. Our solution
    // is a flag that will compress redraws, and will be true by default.
    // Only the last event causes a Redraw, and it will be for the whole
    // canvas.


    if (_compressRedraw)
      {
	if (count != 0)		// ignore all but last expose event
	    return;
	Redraw(0, 0, _width, _height);
      }
    else
      {
	Redraw(x, y , width, height);
      }
  }

//=====================>>> vCanvasPane::EnterFocus <<<========================
  void vCanvasPane::EnterFocus()
  {
    SysDebug(WindowEvents,"vCanvasPane::EnterFocus()\n")
    if (_currentCursor != VC_None)
	SetCursor(_currentCursor);
  }

//=====================>>> vCanvasPane::EnterEV <<<==========================
  void vCanvasPane::EnterEV()
  {
/*
    if (_hasFocus)		// don't double enter focus
	return;
    _hasFocus = 1;

#ifdef Motif
    XmProcessTraversal(_drawWindow,XmTRAVERSE_CURRENT);	// Xm bug, call twice!
    XmProcessTraversal(_drawWindow,XmTRAVERSE_CURRENT);
#endif

    theApp->XCheckSelection();
    EnterFocus();		// call the virtual function
*/
  }

//=====================>>> vCanvasPane::LeaveFocus <<<=======================
  void vCanvasPane::LeaveFocus()
  {
    SysDebug(WindowEvents,"vCanvasPane::LeaveFocus()\n")
  }

//=====================>>> vCanvasPane::LeaveEV <<<==========================
  void vCanvasPane::LeaveEV()
  {
    if (!_hasFocus)		// don't double leave focus
	return;
    _hasFocus = 0;

    LeaveFocus();		// call the virtual function
  }

//=====================>>> vCanvasPane::MouseDownEV <<<=======================
  void vCanvasPane::MouseDownEV(int x, int y, int button)
  {
    // We track mouse down internally - X should do it, but
    // sometimes it seems to get the mouse down withoud sending
    // the event, making MouseMove not work as advertised.
    if (_mouseDown)
	return;
    _mouseDown = 1;
    MouseDown(x,y,button);
  }

//=====================>>> vCanvasPane::MouseUpEV <<<=========================
  void vCanvasPane::MouseUpEV(int x, int y, int button)
  {
    if (!_mouseDown)
	return;
    _mouseDown = 0;
    MouseUp(x,y,button);
  }

//=====================>>> vCanvasPane::MouseMoveEV <<<=======================
  void vCanvasPane::MouseMoveEV(int x, int y, int button)
  {
    if (_mouseDown && button != 0)		// only if down
	MouseMove(x,y,button);
    else
      {
	MouseMotion(x,y);
      }
  }

//=====================>>> vCanvasPane::MouseDown <<<==========================
  void vCanvasPane::MouseDown(int x, int y, int button)
  {
    // Mouse events in vCanvasPane are no-ops.

    SysDebug3(MouseEvents,"vCanvasPane::MouseDown(x:%d,y:%d,btn:%d)\n",x,y,button)
  }

//=====================>>> vCanvasPane::MouseUp <<<============================
  void vCanvasPane::MouseUp(int x, int y, int button)
  {
    // Mouse events in vCanvasPane are no-ops.

    SysDebug3(MouseEvents,"vCanvasPane::MouseUp(x:%d,y:%d,btn:%d)\n",x,y,button)
  }

//=====================>>> vCanvasPane::MouseMove <<<==========================
  void vCanvasPane::MouseMove(int x, int y, int button)
  {
    // Mouse events in vCanvasPane are no-ops.

    SysDebug3(MouseEvents,"vCanvasPane::MouseMove(x:%d,y:%d,btn:%d)\n",x,y,button)
  }

//======================>>> vCanvasPane::FontChanged <<<===========================
  void vCanvasPane::FontChanged(VCONST vFont& vf)
  {
    // Called when the font is changed.

    SysDebug(WindowEvents,"vCanvasPane::FontChanged()\n")
  }

//=======================>>> vCanvasPane::Redraw <<<==========================
  void vCanvasPane::Redraw(int x, int y, int width, int height)
  {
    // Redraw in vCanvasPane is a no-op.
    
#ifdef vDEBUG		// Don't have a SysDebug4 macro, so do it by hand
    if (DebugState.WindowEvents && DebugState.System)
	fprintf(stderr,"vCanvasPane::Redraw(x=%d, y=%d, w=%d, h=%d)\n",x,y,width,height);
#endif

  }

//=====================>>> vCanvasPane::Resize <<<============================
  void vCanvasPane::Resize(int newW, int newH)
  {
    // This is the routine the user will override to intercept size changes

    SysDebug2(WindowEvents,"vCanvasPane::Resize(newW:%d, newH:%d)\n",newW,newH)

  }

//=====================>>> vCanvasPane::CnvKeyInEv <<<============================
  void vCanvasPane::CnvKeyInEv(unsigned int keycode, unsigned int state)
  {
    _parentWin->KeyInEV(keycode, state);
  }

extern "C"
{
  /*
  void CSetFocusChild(Widget w, GtkWidget *focus_widget, gpointer This)
  {
    vCanvasPane *canvas = (vCanvasPane*) This;
    //gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "set-focus-child");
    if (focus_widget && focus_widget != canvas->_drawCanvas)
	gtk_widget_grab_focus((GtkWidget *)canvas->_drawWindow);
  }
  */

    static void CFocusIn(GtkWidget *widget, GdkEventFocus *event)
    {
	GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
    }
    static void CFocusOut(GtkWidget *widget, GdkEventFocus *event)
    {
	GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
    }

void CchangeCB(GtkWidget *w, GdkEventConfigure *ev, gpointer This)
/*
//==============================>>> CchangeCB <<<==============================
  void CchangeCB(Widget w, XtPointer client_data, XtPointer call_data)
*/
  { 
    //	change callback
    //
    // client_data will have the this pointer of our object
    // call_data will point to a structure that handles
    // resize, scrollbars, and exposure

    ((vCanvasPane*)This)->ChangeCB(ev->width, ev->height);
/*
    ((vCanvasPane*)client_data)->ChangeCB();
*/
  }

//==============================>>> CExposeEV <<<==============================
void CExposeEV(GtkWidget *w, GdkEventExpose *ev, gpointer This)
/*
  void CExposeEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    ((vCanvasPane*)This)->ExposeEV(ev->area.x, ev->area.y,
	    ev->area.width, ev->area.height, ev->count);
/*

    //	Exposure event
    //
    // client_data will have the this pointer of our object

    XExposeEvent* xp;

    xp = (XExposeEvent*)event;

    ((vCanvasPane*)client_data)->ExposeEV(xp->x, xp->y, xp->width, xp->height, xp->count);
*/
  }

//=============================>>> CMouseDownEV <<<============================
void CMouseDownEV(GtkWidget *w, GdkEventButton *event, gpointer This)
/*
  void CMouseDownEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    ((vCanvasPane*)This)->MouseDownEV(event->x, event->y, event->button);
    // Mouse down event
/*
    XButtonEvent* xp;

    xp = (XButtonEvent*)event;

    ((vCanvasPane*)client_data)->MouseDownEV(xp->x, xp->y, xp->button);
*/
  }

//==============================>>> CMouseUpEV <<<===========================
void CMouseUpEV(GtkWidget *w, GdkEventButton *event, gpointer This)
/*
  void CMouseUpEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    ((vCanvasPane*)This)->MouseUpEV(event->x, event->y, event->button);
/*
    // Mouse up event
    XButtonEvent* xp;

    xp = (XButtonEvent*)event;

    ((vCanvasPane*)client_data)->MouseUpEV(xp->x, xp->y, xp->button);
*/
  }

//============================>>> CMouseMoveEV <<<==========================
void CMouseMoveEV(GtkWidget *w, GdkEventMotion *event, gpointer This)
/*
  void CMouseMoveEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    int x, y;
    GdkModifierType state;

    if (event->is_hint)
	gdk_window_get_pointer(event->window, &x, &y, &state);
    else {
	x = (int) event->x;
	y = (int) event->y;
	state = (GdkModifierType) event->state;
    }
    int button = 0;		// assume button 1

    if (state & GDK_BUTTON1_MASK)
	button = 1;
    else if (state & GDK_BUTTON2_MASK)
	button = 2;
    else if (state & GDK_BUTTON3_MASK)
	button = 3;

    ((vCanvasPane*)This)->MouseMoveEV(x, y, button);
/*
    // Mouse down event
    XMotionEvent* xp;

    xp = (XMotionEvent*)event;

    int button = 0;		// assume button 1

    if (xp->state & Button1Mask)
	button = 1;
    else if (xp->state & Button2Mask)
	button = 2;
    else if (xp->state & Button3Mask)
	button = 3;

    ((vCanvasPane*)client_data)->MouseMoveEV(xp->x, xp->y, button);
*/
  }

// common to both Motif and Athena

//==============================>>> CEnterEV <<<=============================
void CEnterEV(GtkWidget *w, GdkEvent *event, gpointer This)
/*
  void CEnterEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    //	Enter event
    //
    // client_data will have the this pointer of our object

    ((vCanvasPane*)This)->EnterEV();
  }

//==============================>>> CLeaveEV <<<==============================
void CLeaveEV(GtkWidget *w, GdkEvent *event, gpointer This)
/*
  void CLeaveEV(Widget w, XtPointer client_data, XEvent* event, char *x)
*/
  { 
    //	Leave event
    //
    // client_data will have the this pointer of our object

    ((vCanvasPane*)This)->LeaveEV();
  }

//============================>>> CHJumpProcCB <<<=============================
  void CHJumpProcCB(GtkAdjustment *adj, gpointer This)
/*
  void CHJumpProcCB(Widget w, XtPointer This, XtPointer pc_ptr)
*/
  {
/*
    float percent = *(float*)pc_ptr;	// get the percent back
*/
    ((vCanvasPane*)This)->HJumpProcCB(adj->value);
  }

/*
//============================>>> CHScollProcCB <<<=============================
  void CHScrollProcCB(Widget w, XtPointer This, XtPointer position)
  {
#ifdef Motif
    XmScrollBarCallbackStruct* cbp = (XmScrollBarCallbackStruct*)position;
    switch (cbp->reason)
      {
	case XmCR_DECREMENT:
	case XmCR_PAGE_DECREMENT:
	  {
	    ((vCanvasPane*)This)->HScrollProcCB(-1);
	    break;
	  }
	case XmCR_INCREMENT:
	case XmCR_PAGE_INCREMENT:
	  {
	    ((vCanvasPane*)This)->HScrollProcCB(1);
	    break;
	  }
	case XmCR_VALUE_CHANGED:
	  {
	    ((vCanvasPane*)This)->HJumpProcCB(0.0, cbp->value);
	    break;
	  }
      }
#else
    int pos = (int)position;
    ((vCanvasPane*)This)->HScrollProcCB(pos);
#endif
  }

*/
//============================>>> CJumpProcCB <<<=============================
  void CVJumpProcCB(GtkAdjustment *adj, gpointer This)
  {
    ((vCanvasPane*)This)->VJumpProcCB(adj->value);
  }

/*
//============================>>> CVScollProcCB <<<=============================
  void CVScrollProcCB(Widget w, XtPointer This, XtPointer position)
  {
#ifdef Motif
    XmScrollBarCallbackStruct* cbp = (XmScrollBarCallbackStruct*)position;
    switch (cbp->reason)
      {
	case XmCR_DECREMENT:
	case XmCR_PAGE_DECREMENT:
	  {
	    ((vCanvasPane*)This)->VScrollProcCB(-1);
	    break;
	  }
	case XmCR_INCREMENT:
	case XmCR_PAGE_INCREMENT:
	  {
	    ((vCanvasPane*)This)->VScrollProcCB(1);
	    break;
	  }
	case XmCR_VALUE_CHANGED:
	  {
	    ((vCanvasPane*)This)->VJumpProcCB(0.0, cbp->value);
	    break;
	  }
      }
#else
    int pos = (int)position;
    ((vCanvasPane*)This)->VScrollProcCB(pos);
#endif
  }

*/
}
