//===============================================================
// vx2vkey.cxx - routines for converting V key codes
//
// Copyright (C) 1995,1996  Bruce E. Wampler
//
// This file is part of the V C++ GUI Framework, and is covered
// under the terms of the GNU Library General Public License,
// Version 2. This library has NO WARRANTY. See the source file
// vapp.cxx for more complete information about license terms.
//===============================================================

#include <v/v_defs.h>
#include <v/vkeys.h>
#include <v/vgdk2vkey.h>
#include <gdk/gdkkeysyms.h>


  static vKey GdktoV[] =
  {
    GDK_BackSpace, vk_BackSpace,  GDK_Tab, vk_Tab,  GDK_Linefeed, vk_Linefeed,
    GDK_Return, vk_Return,  GDK_Pause, vk_Pause,  GDK_Escape, vk_Escape,
    GDK_Delete, vk_Delete,  GDK_Home, vk_Home,  GDK_Left, vk_Left,
    GDK_Insert, vk_Insert,
    GDK_Up, vk_Up,  GDK_Right, vk_Right,  GDK_Down, vk_Down,  GDK_Prior, vk_Page_Up,
    GDK_Next, vk_Page_Down,  GDK_End, vk_End,  GDK_Num_Lock, vk_Num_Lock,
    GDK_KP_Enter, vk_KP_Enter,  GDK_KP_Home, vk_KP_Home,  GDK_KP_Left, vk_KP_Left,
    GDK_KP_Up, vk_KP_Up,  GDK_KP_Right, vk_KP_Right,  GDK_KP_Down, vk_KP_Down,
    GDK_KP_Page_Up, vk_KP_Page_Up,  GDK_KP_Page_Down, vk_KP_Page_Down,
    GDK_KP_End, vk_KP_End,  GDK_KP_Insert, vk_Insert,  GDK_KP_Delete, vk_KP_Delete,
    GDK_KP_Equal, vk_KP_Equal,  GDK_KP_Multiply, vk_KP_Multiply,
    GDK_KP_Add, vk_KP_Add,  GDK_KP_Subtract, vk_KP_Subtract,
    GDK_KP_Decimal, vk_KP_Decimal,  GDK_KP_Divide, vk_KP_Divide,
    GDK_KP_0, vk_KP_0,  GDK_KP_1, vk_KP_1,  GDK_KP_2, vk_KP_2,  GDK_KP_3, vk_KP_3,
    GDK_KP_4, vk_KP_4,  GDK_KP_5, vk_KP_5,  GDK_KP_6, vk_KP_6,  GDK_KP_7, vk_KP_7,
    GDK_KP_8, vk_KP_8,  GDK_KP_9, vk_KP_9,  GDK_F1, vk_F1,  GDK_F2, vk_F2,
    GDK_F3, vk_F3,  GDK_F4, vk_F4,  GDK_F5, vk_F5,  GDK_F6, vk_F6,  GDK_F7, vk_F7,
    GDK_F8, vk_F8,  GDK_F9, vk_F9,  GDK_F10, vk_F10,  GDK_F11, vk_F11,  GDK_F12, vk_F12,

    // The following might only apply to Linux, but there probably won't
    // be many using F12-F20 anyway. Linux maps Shift-F1 to F13.
  
    GDK_F13, vk_F1,  GDK_F14, vk_F2,  GDK_F15, vk_F3,  GDK_F16, vk_F4,  GDK_F17, vk_F5,
    GDK_F18, vk_F6,  GDK_F19, vk_F7,  GDK_F20, vk_F8,  
   
    // The following are modifer keys pressed by themselves
  
    GDK_Shift_L, vk_Shift,  GDK_Shift_R, vk_Shift,  GDK_Control_L, vk_Control,
    GDK_Control_R, vk_Control,  GDK_Caps_Lock, vk_Caps_Lock,
    GDK_Alt_L, vk_Alt,  GDK_Alt_R, vk_Alt, GDK_Mode_switch, vk_Alt, 
    0, 0
  };


//===============================>>> MapXtoVkey <<<================================
  vKey MapGdktoVkey(unsigned int Gdkkey, unsigned int& shift)
  {
    // Map an X Window System keyvalue to the internal code used by V

    unsigned int tmp;
    vKey vtmp;

    if (Gdkkey > 0 && Gdkkey <= 255)	// Normal & ISO ascii character
      {
	if (shift == 0)			// no ctrl or alt or shift, so return
	    return Gdkkey;

	if (shift == VKM_Shift)		// only shifted - kill the flag
	  {
	    shift = 0;			// kill the shift flag
	    if (Gdkkey == GDK_Tab)
		return vk_BackTab;	// special case
	    return Gdkkey;		// return actual value
	  }
	else if (shift == VKM_Ctrl || shift == (VKM_Ctrl | VKM_Shift))
	// control-X or ctrl-shift-X
	  {
	    if (Gdkkey >= '@' && Gdkkey <= '_')
	      {
		shift &= VKM_Shift;		// kill the ctrl flag
		vtmp = Gdkkey - '@';	// normal range
	      }
	    else if (Gdkkey >= '`' && Gdkkey <= '~')
	      {
		shift &= VKM_Shift;		// kill the ctrl flag
		vtmp = Gdkkey - '`';	// normal range
	      }
	    else
		vtmp = Gdkkey;		// just let it map;

	    return vtmp;
	  }
	// else it is combination of Alt, Shift, cTRL
	return Gdkkey;
      }

    // To here, then need to map
    for (int ix = 0 ; GdktoV[ix] != 0 ; ix += 2 )
      {
	// Search the XtoV table
	if (GdktoV[ix] == Gdkkey)
	  {
	    return GdktoV[ix+1];
	  }
      }
    return vk_None;
  }
