//===============================================================
// vStatusPane - the status bar pane class used by the vWindow class
//
// Copyright (C) 1995,1996  Bruce E. Wampler
//
// This file is part of the V C++ GUI Framework, and is covered
// under the terms of the GNU Library General Public License,
// Version 2. This library has NO WARRANTY. See the source file
// vapp.cxx for more complete information about license terms.
//===============================================================

#include <v/vxutil.h>
#include <v/vstatusp.h>
/*
#include <v/vctlclrs.h>
*/

#include <v/vwindow.h>		// we are part of a window
#include <v/vfont.h>
#include <v/vapp.h>

//===================>>> vStatusPane::vStatusPane <<<====================
  vStatusPane::vStatusPane(vStatus* s_bar) : vPane(P_Status)
  {
    // The main job of this constructor is to add the list of
    // status boxes to our internal data structure.  We will not
    // realize the status boxes until the initialize routine is called.

    StatusList* temp;

    SysDebug(Constructor,"vStatusPane::vStatusPane() constructor\n")

    _statusList = 0;			// empty list

    // we are going to reverse these as we realize the list,
    // so work the list in reverse order by finding the end first

    int limit;
    for (limit = 0 ; s_bar[limit].label ; ++limit)
	;

    // now, work the list backwards

    for (int ix = limit - 1 ; ix >= 0 ; --ix)
      {
	temp = new StatusList;		// a new list cell
	temp->statusPtr = &s_bar[ix];   // use the static definition of caller
	temp->nextSL = _statusList;	// link in
	_statusList = temp;		// to front of list
      }
  }

//===================>>> vStatusPane::vStatusPane <<<====================
  vStatusPane::vStatusPane(const vStatusPane& b) : vPane(b)
  {
    vSysError("vStatusPane: V Semantics do not allow copy constructors.");
  }

//===================>>> vStatusPane::~vStatusPane <<<====================
  vStatusPane::~vStatusPane()
  {

    SysDebug(Destructor,"vStatusPane::~vStatusPane() destructor\n")

    StatusList* temp;

    for (StatusList* sl = _statusList ; sl != 0 ;)
      {
	temp = sl;		// now need to free the status list
	sl = sl->nextSL;
	delete temp;
      }
  }

//===================>>> vStatusPane::ShowPane <<<========================
  void vStatusPane::ShowPane(int OnOrOff) VCONST
  {
    /* map / unmap or show / hide ??? XXX */
    if (OnOrOff)
      {
	gtk_widget_show(GTK_WIDGET(_statusBar));
      }
    else
      {
	gtk_widget_hide(GTK_WIDGET(_statusBar));
      }
  }

//===================>>> vStatusPane::SetPaneValue <<<========================
  void vStatusPane::SetPaneValue(ItemVal id, int val, ItemSetType setType)
  {
    // set any matching status boxes in list to val
 
    
    if (setType != Sensitive)
	return;
    for (StatusList* sl = _statusList ; sl != 0 ; sl = sl->nextSL )
      {
	if ((sl->statusPtr)->statId == id)
	  {
	    (sl->statusPtr)->sensitive = val;
	    gtk_widget_set_sensitive(GTK_WIDGET(sl->w), val);
	  }
      }
  }

//====================>>> vStatusPane::SetPaneString <<<======================
  void vStatusPane::SetPaneString(ItemVal id, VCONST char* str)
  {
    // change the label in a status
    for (StatusList* sl = _statusList ; sl != 0 ; sl = sl->nextSL )
      {
	if ((sl->statusPtr)->statId == id)
	  {
	    gtk_label_set(GTK_LABEL(sl->w), str);
	    if ((sl->statusPtr)->width > 0)	// they specified width
	      {
		gtk_widget_set_usize(GTK_WIDGET(sl->w), (sl->statusPtr)->width, -2);
	      }
	  }
      }
  }

//====================>>> vStatusPane::initialize <<<=======================
  void vStatusPane::initialize(vWindow* pWin, Widget pWidget,
	Widget drawCanvas,  Widget drawArea)
  {
     // now, build the status bar items in the widget provided

    vPane::initialize(pWin, pWidget);	// initialize base class

/*
    // Create pixmap
    if (statbar_bitmap == 0)		// haven't built the pixmap yet
      {
	statbar_bitmap = XCreatePixmapFromBitmapData(
	    theApp->display(),				// our display
	    DefaultRootWindow(theApp->display()),	// a Drawable
	    statbar_bits,		// the statbar bitmap data
	    statbar_width,		// from the file
	    statbar_height,
	    (unsigned long) theApp->Xfg(),	// Forground color
	    (unsigned long) theApp->Xbg(),	// Background color
	    DefaultDepth(theApp->display(),DefaultScreen(theApp->display())));
      }
*/

    // Now the status bar that will have status boxes
    _statusBar = gtk_hbox_new(FALSE, 4);
    gtk_container_border_width(GTK_CONTAINER(_statusBar), 4);
    gtk_box_pack_start(GTK_BOX(_baseW), _statusBar, FALSE, FALSE, 0);
    gtk_widget_show(_statusBar);

/*
    // set the background pattern
    if (theApp->Xdepth() <= 1)
        XtVaSetValues(_statusBar,         // the widget to set
          XtNbackgroundPixmap, statbar_bitmap, NULL);
    else
        XtVaSetValues(_statusBar,         // the widget to set
          XtNbackground, _vStatusBarBG,NULL);
*/

    for (StatusList* sl = _statusList ; sl != 0 ; sl = sl->nextSL )
      {
	int border = ((sl->statusPtr)->attrs & CA_NoBorder) ? 0 : 1;	// border?
	Widget tmpW = _statusBar;
	if (border > 0) {
	    tmpW = gtk_frame_new(NULL);
	    gtk_container_add(GTK_CONTAINER(_statusBar), tmpW);
	    gtk_widget_show(tmpW);
	}
	sl->w = gtk_label_new((sl->statusPtr)->label);
	gtk_container_add(GTK_CONTAINER(tmpW), sl->w);
	gtk_widget_show(sl->w);
	if (!(sl->statusPtr)->sensitive)
	    gtk_widget_set_sensitive(GTK_WIDGET(sl->w), FALSE);

	if ((sl->statusPtr)->width > 0)		// they specified width
	  {
	    gtk_widget_set_usize(GTK_WIDGET(sl->w), (sl->statusPtr)->width, -2);
          }
      }
  }
