//===============================================================
// vtextinc.xxx	- TextIn Cmd
//
// Copyright (C) 1995,1996  Bruce E. Wampler
//
// This file is part of the V C++ GUI Framework, and is covered
// under the terms of the GNU Library General Public License,
// Version 2. This library has NO WARRANTY. See the source file
// vapp.cxx for more complete information about license terms.
//===============================================================

#include <v/vxutil.h>	// Motif/Athena mapping
#include <v/vapp.h>
#include <v/vtextinc.h>	// our definitions
#include <v/vcmdprnt.h>	// a command parent

//=====================>>> vTextInCmd::vTextInCmd <<<=======================
  vTextInCmd::vTextInCmd(vCmdParent* dp, CommandObject* co) :
    vCmd(dp, co)
  {
    initialize();
  }

//=====================>>> vTextInCmd::~vTextInCmd <<<=======================
  vTextInCmd::~vTextInCmd()
  {

    SysDebug(Destructor,"vTextInCmd::~vTextInCmd() destructor\n")

  }

//=====================>>> vTextInCmd::initialize <<<=======================
  void vTextInCmd::initialize(void)
  {
    // build a TextIn command for use in a parent window

    SysDebug(Constructor,"vTextInCmd::vTextInCmd() constructor\n")

    CopyToLocal();		// make local copies

    int defLen = strlen(_title);	// length of supplied string

    int defWidth;

    if (dlgCmd->size <= 0)		// BEW: 7/24/96 - for V 1.13
      {
	defWidth = (dlgCmd->attrs & CA_Large) ? 350 : 200;

	if (dlgCmd->attrs & CA_Small)	// allow small, too.
	    defWidth = 100;
      }
    else
      {
	defWidth = dlgCmd->size * 
		    gdk_text_width(wParent->style->font, "M", 1) + 2;
/*
#ifdef Motif
	defWidth = (dlgCmd->size+2) * XTextWidth(theApp->_XDefaultFont, "M",1);
#else
	defWidth = dlgCmd->size * XTextWidth(theApp->_XDefaultFont, "M",1) + 2;
#endif
*/
	if (defWidth > 800)		// Sanity check
	    defWidth = 500;
      }

    wBox = 0;

    int map = !(dlgCmd->attrs & CA_Hidden);

    wCmd = gtk_entry_new();
    if (map) gtk_widget_show(wCmd);
    gtk_widget_set_usize(wCmd, defWidth, -2);
    gtk_entry_set_text(GTK_ENTRY(wCmd), _title);
    gtk_signal_connect(GTK_OBJECT(wCmd), "activate", 
	GTK_SIGNAL_FUNC(CTextCB), (gpointer)this);

    if (!_Sensitive)
	gtk_widget_set_sensitive(wCmd, FALSE);
    gtk_entry_set_visibility(GTK_ENTRY(wCmd), !(dlgCmd->attrs & CA_Password));
/*

    wCmd = XtVaCreateManagedWidget(
	"vTextIn",		// name
	asciiTextWidgetClass,	// class
	wParent,		// parent
	XtNstring, _title,
	XtNeditType, XawtextEdit,	// editable
	XtNinsertPosition, defLen,
	XtNresizable, True,
	XtNwidth, defWidth,
	XtNmappedWhenManaged, map,
	XtNecho, !(dlgCmd->attrs & CA_Password),
	NULL);

    // kill off the search and return functions
    XtTranslations txtT =
      XtParseTranslationTable("<Key>Return: CtextReturnA()\n\
	Ctrl<Key>R: no-op(RingBell)\n\
	Ctrl<Key>S: no-op(RingBell)\n");

    XtOverrideTranslations(wCmd, txtT);
    XtAppAddActions(theApp->appContext(), actions, XtNumber(actions));
*/

/*
    // Need to set to insensitive if it is set insensitive
    if (!_Sensitive)
      {
	XtVaSetValues(wCmd,		// the widget to set
	    Nsensitive, 0,		// change to insensitive
	    NULL);
      }
*/
  }

//=====================>>> vTextInCmd::GetTextIn <<<=======================
  int vTextInCmd::GetTextIn(ItemVal id, char* str, int maxlen) VCONST
  {
    // recover the data from a TextInCmd - return text + length

    gchar *text = gtk_entry_get_text(GTK_ENTRY(wCmd));

    // now, copy the string from the widget to the user
    // we will take account for length, too.
    char* toP = str;
    int l = 0;				// track the length

    for (char* fromP = (char*)text ; *fromP && l < maxlen ; ++l)
      {
	*toP++ = *fromP++;
      }
    *toP = 0;			// terminate the string
    return l;
  }

//================>>> vTextInCmd::SetCmdVal <<<============================
  void vTextInCmd::SetCmdVal(ItemVal val, ItemSetType st)
  {

    SysDebug1(Misc,"vTextInCmd::SetCmdVal(val:%d)\n",val)

    if (st == Sensitive)
      {
	_Sensitive = val;		// set
	gtk_widget_set_sensitive(wBox, val);
      }
    else if (st == Hidden)		// hide or unhide
      {
	if (val)
	  {
	    gtk_widget_hide(wCmd);
	  }
	else
	  {
	    gtk_widget_show(wCmd);
	  }
      }
    else
      {
	SysDebug2(BadVals,"vTextInCmd::SetCmdVal(val:%d,type:%d) - BAD TYPE\n",val,st)
      }

  }

//================>>> vTextInCmd::SetCmdStr <<<============================
  void vTextInCmd::SetCmdStr(VCONST char* str)
  {

    SysDebug1(Misc,"vTextInCmd::SetCmdStr(str:%s)\n",str)

    int defLen = strlen(str);	// length of supplied string

    gtk_entry_set_text(GTK_ENTRY(wCmd), str);
    gtk_entry_set_position(GTK_ENTRY(wCmd), defLen);
/*
    XtVaSetValues(wCmd,		// the widget to set
#ifdef Motif
	XmNvalue, str,
#else
	XtNstring, str,
#endif
	NULL);

    XtVaSetValues(wCmd,		// the widget to set
	XtNinsertPosition, defLen,
	NULL);
*/
  }

extern "C"
{
//==========================>>> CtextReturnA <<<=============================
  void CTextCB(GtkEntry *w, gpointer This)
  {
    // Make the Return key return from the dialog if there is a default cmd.

    // _TList is a static, so we can get the this back by looking
    // for the proper widget.

    ((vTextInCmd*) This)->textReturn();

  }
}

//=====================>>> vTextInCmd::textReturn <<<=======================
  void vTextInCmd::textReturn(void)
  {
    //@@@ This will interact with the Dialog to return if there
    //	was a default button.
    SysDebug(Misc,"vTextInCmd::textReturn\n")

    _parentWin->ProcessCmd(_cmdId, _retVal, C_TextIn);

  }
